/****************************************************************************
*
*     Copyright (c) 2003 Dave Hylands
*           All Rights Reserved
*
*   Permission is granted to any individual or institution to use, copy, or
*   redistribute this software so long as it is not sold for profit, and that
*   this copyright notice is retained.
*
****************************************************************************/
/**
*
*   @file   Tachometer.cpp
*
*   @brief  Implementation of the Tachometer class
*
****************************************************************************/

// ---- Include Files -------------------------------------------------------

#include <stdio.h>
#include <assert.h>

#include "Log.h"
#include "Tachometer.h"

#include <68332/isr.h>
#include <68332/tpu.h>
#include <68332/pta.h>

// ---- Public Variables ----------------------------------------------------
// ---- Private Constants and Types -----------------------------------------

// ---- Private Variables ---------------------------------------------------
// ---- Private Function Prototypes -----------------------------------------

#define TACH_DEBUG_ENABLED 1
#if TACH_DEBUG_ENABLED
#   define TACH_DEBUG(x)  LOG(x)
#else
#   define TACH_DEBUG(x)
#endif

// ---- Functions -----------------------------------------------------------

/**
 * @addtogroup Peripheral
 * @{
 */

//***************************************************************************
/**
*   Constructor
*/

Tachometer::Tachometer( const char *name )
{
    m_name = name;
    m_channel = NULL;
    m_divPerRev = 1;

    m_divAggr = 0;
    m_divOffset = 0;

    for ( int i = 0; i < HistoryLen; i++ ) 
    {
        m_divCount[ i ] = 0;
    }
}

//***************************************************************************
/**
*   Destructor
*/

Tachometer::~Tachometer()
{
    if ( m_channel != NULL )
    {
        tpu_release_channel( 0, m_channel->channel_num );
    }
}
    
//***************************************************************************
/**
*  Init - Initialize the TPU pins
*/

void Tachometer::Init( int pin )
{
    if (( m_channel = tpu_request_channel( 0, pin )) == NULL )
    {
        LOG_ERROR(( "Unable to get TPU pin %d for %s Tachometer", pin, m_name ));
        return;
    }
    m_channel->userData = this;

    pta_init( m_channel, PTA_CHAN_TYPE_PERIOD_RISING_EDGE );
    
    install_isr( tpu_get_interrupt_vector( m_channel ), (void(*)())InterruptHandler );

    tpu_set_CIER( m_channel, 1 );

} // Init

//***************************************************************************
/**
*   InterruptHandler - gets called to process interrupts for the Tachometer
*                      channel.
*/

// static
void Tachometer::InterruptHandler( unsigned int vectorNum )
{
    uint32      currCount;                                  
    uint8       channelNum = (uint8)( vectorNum & 0x0F );
    Tachometer *tach = (Tachometer *)tpu_get_channel_user_data( 0, channelNum );

    if (( tach == NULL ) || !tpu_get_CISR( tach->m_channel ))
    {
        // Something weird is happening.

        return;
    }

    currCount = pta_result( tach->m_channel );

    // Update the history information

    tach->m_divAggr -= tach->m_divCount[ tach->m_divOffset ];
    tach->m_divCount[ tach->m_divOffset ] = currCount;
    tach->m_divAggr += currCount;

    if ( ++tach->m_divOffset >= HistoryLen )
    {
        tach->m_divOffset = 0;
    }

    // Clear the interrupt

    tpu_clear_CISR( tach->m_channel );

} // InterruptHandler

//***************************************************************************
/**
*   Rpm - Returns the revolutions per minute being detected by the tachomter.
*         for a single revolution.
*
*   @returns    RPMs
*/

unsigned int Tachometer::Rpm()
{
    uint32  count = m_divAggr / HistoryLen;

    if (( count == 0 ) || ( m_divPerRev == 0 ))
    {
        // Prevent unpredictable divide by zero

        return 0;
    }

    // iprintf( "Tach freq = %lu count = %d, divPerRev = %d\n", tpu_get_tcr1_freq( 0 ), count, m_divPerRev );

    uint32  rpm = ( tpu_get_tcr1_freq( 0 ) * 60 ) / ( count * m_divPerRev );

    return rpm;

} // Rpm

//***************************************************************************
/**
*   SetDivisionsPerRev - Sets the number of bands that need to be counted
*                        for a single revolution.
*
*   @param   divPerRev   (in)  Number of divions required for one revolution.
*/

void Tachometer::SetDivisionsPerRev( unsigned divPerRev )
{
    TACH_DEBUG(( "%s Tachometer: SetDivisionsPerRev( %u )\n", m_name, divPerRev ));

    m_divPerRev = divPerRev;

} // SetDivisionsPerRev

/** @} */

