/****************************************************************************
*
*  Program for Twister, a Line Maze robot.
*
****************************************************************************/

#include <stdio.h>
#include <68332/isr.h>
#include <68332/qsm.h>
#include <68332/qsm_polled.h>
#include <68332/sim.h>
#include <68332/tpu.h>
#include <68332/pta.h>
#include <68332/qompwm.h>
#include <68332/mrm/led.h>
#include <68332/mrm/lcd.h>
#include <../libsim/68332/sim_regs.h>
#include <assert.h>

#include "Exception.h"
#include "Tachometer.h"
#include "Tick.h"

#define BAUD_RATE 0     // 0 means use the current baud rate

int gTickFlag = 0;
int gTickOverrun = 0;
volatile int gTickCounter  = 0;

void TickHandler( void );

TPUChannel  *gPwmChannel;


Tachometer  gTach( "Tach" );

typedef unsigned int    sample_t;

#include "68332/sim.h"

void ResetProcessor( void )
{
    SIM_T *sim = (SIM_T *)SIM_BASE;

    // First thing is to turn off all interrupt sources. This test program
    // sets up the PITR

//    sim->PICR.BITS.PIRQL = 0;

    // Turn on the Halt Monitor. This will cause the processor to reset if
    // a halt is detected.

    sim->SYPCR.BITS.HME = 1;
    sim->SYPCR.BITS.BME = 1;

    // Set the stack pointer to an odd address and then do a word access to
    // an oddd address. This will cause a double fault, which will cause
    // the processor to HALT.

    asm( "    move.l #1,%a7" );
    asm( "    move.w (%a7),%d0" );
}

void ResetProcessor2( void)
{
    unsigned int *p = (unsigned int *)0x90000;

    iprintf( "Sig = 0x%08.8x\n", *p++ );
    iprintf( "Stk = 0x%08.8x\n", *p++ );
    iprintf( "Adr = 0x%08.8x\n", *p++ );
    iprintf( "Lst = 0x%08.8x\n", *p++ );

//    asm( "    RESET" );

//    iprintf( "After Reset\n" );

    asm( "    ori.w  #0x0700,%sr" );
    asm( "    move.l #0x90000,%a3" );
    asm( "    move.l (%a3)+,%d0" );
    asm( "    move.l (%a3)+,%a7" );
    asm( "    move.l (%a3)+,%a0" );
    asm( "    jmp (%a0)" );
}



int main( void )
{
    int  nextTick;
    int  ledState = 0;
    SIM_T *sim = (SIM_T *)SIM_BASE;

    ExceptionInit();
    
    set_processor_speed( CLOCK25MHZ );
    
    /*
     * since sciinit is required for all programs that do i/o
     * perhaps it should be in mrm_crt0.S also?  It would still
     * be available if the user wanted to change baud rate.
     */
    
    sciinit( BAUD_RATE );
    
    led_init();
    lcd_init( 2, 24, 0, FALSE );

    set_int_level( 7 );
    {
        char   *s;

        set_int_level( 7 );

        s = "*** Test ***\r\n";
        
        while ( *s != '\0' )
        {
            qsm_polled_outbyte( *s++ );
        }
    }
    set_int_level( 0 );

    
    tpu_init( 0 );
    tpu_init_interrupts( 0, 0x0E, 5, 0x60 >> 4 );

    gTach.Init( 1 );
    gTach.SetDivisionsPerRev( 6 );

    {
        /*
         * Setup a PWM signal for testing purposes.
         */

        if (( gPwmChannel = tpu_request_channel( 0, 0 )) != NULL )
        {
            uint32 pulseWidth = tpu_get_tcr1_freq( 0 ) / 240; // 240 Hz
            uint16 hiTime = pulseWidth / 2;
            uint16 loTime = pulseWidth / 2;
            
            qompwm( gPwmChannel, hiTime, loTime );

            //iprintf( "pulseWidth = %u\n", pulseWidth );
        }
    }

    /*
     * Initialize the periodic timer. A value of 8 should cause a 1024 Hz
     * interrupt.
     *
     * With PTP == 0, you get interrupted 8192/PITM times per second.
     * With PTP == 1, you get interrupted   16/PITM times per second.
     */
    
    gTickCounter = 0;
    install_isr( 64, TickHandler );
    sim->PITR.BITS.PTP   = 0;
    sim->PITR.BITS.PITM  = 8192 / TICK_HZ;
    sim->PICR.BITS.PIRQL = 6;
    sim->PICR.BITS.PIV   = 64;

    while ( 1 )
    {
        nextTick = gTickCounter >> 8;
        while ( nextTick == ( gTickCounter >> 8 ))
        {
            ;
        }
        
        if ( ledState )
        {
            int min = nextTick / 120;
            int sec = ( nextTick / 2 ) % 60;
            unsigned int rpm = gTach.Rpm();
            
            iprintf(    "\r%02d:%02d  %4d RPM   ", min, sec, rpm );
            lcd_printf( "\f%02d:%02d  %4d RPM   ", min, sec, rpm );
            
            fflush( stdout );
            
            led_on( GreenLED );
            led_off( RedLED );

            if ( sec > 10 )
            {
                ResetProcessor();
            }
        }
        else
        {
            led_off( GreenLED );
            led_on( RedLED );
        }
        ledState = !ledState;
   }

   exit( 0 );
}

/****************************************************************************
*
*  Called for each tick of the Periodic Interval Timer.
*
****************************************************************************/

void TickHandler( void )
{

   if( gTickFlag )
   {
      gTickOverrun++;
      return;
   }
   gTickFlag = 1;

   gTickCounter++;

   gTickFlag = 0;
}

